%define aix_version 6.1.0.0

%define major 10
%define minor 3
%define fix 0

%define libgomp_sover 1
%define libstdcxx_sover 6
%define libgfortran_sover 5
%define libatomic_sover 1
%define libgcc_sover 1

Name: gcc%{major}
Version: %{major}.%{minor}.%{fix}
Release: 9
License: GPL-3.0-or-later
URL: http://gcc.gnu.org/
Summary: GNU Compiler Collection

BuildRequires: mpfr-devel
BuildRequires: gmp-devel
BuildRequires: mpc-devel
BuildRequires: isl-devel
BuildRequires: zlib-devel
BuildRequires: libtool
BuildRequires: gawk
BuildRequires: bison
BuildRequires: flex
BuildRequires: expect
BuildRequires: texinfo
BuildRequires: libiconv-devel
BuildRequires: pase-build-tools

Requires: gcc%{major}-cpp = %{version}-%{release}
Requires: libgcc_s%{libgcc_sover} >= %{version}-%{release}
Requires: libatomic%{libatomic_sover} >= %{version}-%{release}
Requires: libgomp%{libgomp_sover} >= %{version}-%{release}
Requires: libstdcplusplus%{libstdcxx_sover} >= %{version}-%{release}

Source0: https://ftp.gnu.org/gnu/gcc/gcc-%{version}/gcc-%{version}.tar.xz

Patch1:   upstream-backport-gcc11-aix-64bit-gcc.patch
Patch3:   gcc10_makefile.patch
Patch5:   gcc10-fortran-undefined-symbol.patch
Patch9:   gcc10-pase-config.patch
Patch10:  gcc10-m64-opt-support.patch

%description
The gcc package contains the GNU Compiler Collection version %{version}.
You'll need this package in order to compile C code.


%package -n gcc%{major}-cpp
Summary: The C Preprocessor
Requires: libgcc_s%{libgcc_sover} >= %{version}-%{release}
Requires: libstdcplusplus%{libstdcxx_sover} >= %{version}-%{release}

%description -n gcc%{major}-cpp
Cpp is the GNU C-Compatible Compiler Preprocessor.
Cpp is a macro processor which is used automatically
by the C compiler to transform your program before actual
compilation. It is called a macro processor because it allows
you to define macros, abbreviations for longer
constructs.

The C preprocessor provides four separate functionalities: the
inclusion of header files (files of declarations that can be
substituted into your program); macro expansion (you can define macros,
and the C preprocessor will replace the macros with their definitions
throughout the program); conditional compilation (using special
preprocessing directives, you can include or exclude parts of the
program according to various conditions); and line control (if you use
a program to combine or rearrange source files into an intermediate
file which is then compiled, you can use line control to inform the
compiler about where each source line originated).

You should install this package if you are a C programmer and you use
macros.


%package -n gcc%{major}-cplusplus
Summary: C++ support for GCC

Requires: gcc10 = %{version}-%{release}
Requires: libgcc_s%{libgcc_sover} >= %{version}-%{release}
Requires: libstdcplusplus%{libstdcxx_sover} >= %{version}-%{release}
Requires: libstdcplusplus%{libstdcxx_sover}-%{major}-devel = %{version}-%{release}

%description -n gcc%{major}-cplusplus
This package adds C++ support to the GNU Compiler Collection.
It includes support for most of the current C++ specification,
including templates and exception handling.


%package -n libgcc_s%{libgcc_sover}
Summary: GCC support library

%description -n libgcc_s%{libgcc_sover}
Provides libgcc_s.so.%{libgcc_sover}


%package -n libatomic%{libatomic_sover}
Summary: GCC support library

%description -n libatomic%{libatomic_sover}
Provides libatomic.so.%{libatomic_sover}


%package -n libgomp%{libgomp_sover}
Summary: GNU OpenMP library

%description -n libgomp%{libgomp_sover}
The libgomp library provides OpenMP support for GCC.


%package -n libstdcplusplus%{libstdcxx_sover}
Summary: GNU Standard C++ Library

%description -n libstdcplusplus%{libstdcxx_sover}
The libstdcplusplus package contains a rewritten standard compliant GCC Standard
C++ Library.


%package -n libstdcplusplus%{libstdcxx_sover}-%{major}-devel
Summary: Development files for libstdcplusplus%{libstdcxx_sover}

%description -n libstdcplusplus%{libstdcxx_sover}-%{major}-devel
Development files for libstdcplusplus%{libstdcxx_sover}


%package -n gcc%{major}-gfortran
Summary: Fortran 95 support
Requires: gcc10 = %{version}-%{release}
Requires: libgfortran%{libgfortran_sover} >= %{version}-%{release}

%description -n gcc%{major}-gfortran
The gcc-gfortran package provides support for compiling Fortran 95
programs with the GNU Compiler Collection.

%package -n libgfortran%{libgfortran_sover}
Summary: Fortran 95 libraries

%description -n libgfortran%{libgfortran_sover}
Summary: GNU Fortran libraries


%prep
%setup -q -n gcc-%{version}
%patch1 -p1
%patch3 -p1
%patch5 -p1
%patch9 -p1
%patch10 -p1

# Hack to remove -G option when using --with-aix-soname=svr4
# This produces the following changes
#
#        if test "$aix_use_runtimelinking" = yes; then
#-         shared_flag="$shared_flag "'${wl}-G'
#+         true
#        fi
#
# (We could remove the whole if block, but sed really only operates on lines)
#
# The shared_flag variable is used to set the archive_expsym_cmds variable,
# which is used in the libtool script generated by config.status + ltmain.sh
# With this and Patch7 to remove the GCC-specific handling of libgcc_s to match
# autotools, we should be good.
#
# Ultimately, this should get fixed by libtool, but that brings a whole host
# of other problems/dependencies.

find . -name configure | \
  xargs sed -i "s/shared_flag=\"\\\$shared_flag \"'\\\${wl}-G'/true/"

# Remove t-aix files that build fat aix libraries
# we dont care to ship these .a libs anyways
# similar to what was done in  gcc10-no-aix-style-fat-libs.patch
# instead here we simply remove these files
rm libatomic/config/t-aix
rm libgfortran/config/t-aix
rm libgomp/config/t-aix
rm libstdc++-v3/config/os/aix/t-aix

%build

cat <<EOF > ar
#!/QOpenSys/usr/bin/sh
exec /QOpenSys/usr/bin/ar -X64 "\$@"
EOF
chmod +x ar

unset OBJECT_MODE

export LIBPATH=%{_libdir}:/QOpenSys/usr/lib
export CONFIG_SHELL="%{_bindir}/bash"
export CONFIGURE_ENV_ARGS=$CONFIG_SHELL
export PATH=$PWD:$PATH
export SED="%{_bindir}/sed"
export RM="%{_bindir}/rm -f"
export M4="%{_bindir}/m4"

RPM_FLAGS="-maix64 -Wl,-brtl -Wl,-blibpath:$LIBPATH"

CC=%{_bindir}/gcc \
CXX=%{_bindir}/g++ \
%configure \
  CFLAGS="$RPM_FLAGS" \
  CXXFLAGS="$RPM_FLAGS" \
  LDFLAGS="$RPM_FLAGS" \
  --program-suffix=-%{major} \
  --enable-languages=c,c++,fortran \
  --with-gmp=%{_prefix} \
  --with-mpfr=%{_prefix} \
  --with-mpc=%{_prefix} \
  --with-isl=%{_prefix} \
  --with-system-zlib \
  --host=powerpc-ibm-aix%{aix_version} \
  --build=powerpc-ibm-aix%{aix_version} \
  --with-cpu=default64 \
  --with-pkgversion="IBM i" \
  --disable-werror \
  --disable-libcc1 \
  --disable-nls \
  --with-cloog=no \
  --with-ppl=no \
  --disable-plugin \
  --disable-cet \
  --enable-__cxa_atexit \
  --disable-libstdcxx-pch \
  --disable-rpath \
  --enable-libstdcxx-filesystem-ts \
  --enable-decimal-float=dpd \
  --enable-version-specific-runtime-libs \
  --with-specs=%{!maix64:-maix64} \
  --disable-bootstrap \
  --enable-shared \
  --disable-static \
  --with-aix-soname=svr4 \
  --with-gcc-major-version-only \
  # end

export     OPT_FLAG="-O2 -D_LARGE_FILES"
export	BOOT_CFLAGS="${OPT_FLAG}"
export	  LT_CFLAGS="${OPT_FLAG}"
export	     CFLAGS="${OPT_FLAG}"
export	   CXXFLAGS="${OPT_FLAG}"
export	  LIBCFLAGS="${OPT_FLAG}"
export	LIBCXXFLAGS="${OPT_FLAG} -fno-implicit-templates"

echo "BUILD ENVIRONMENT:"
/usr/bin/env | /usr/bin/sort

%make_build

%install

%make_install

cd %{buildroot}%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/include/c++/powerpc-ibm-aix%{aix_version}
cp -f -R pthread/* .
rm -r pthread

# create a symlink to resolve to multilib pthread path
# without this symlink include files are not resolved e.g:
# /QOpenSys/pkgs/lib/gcc/powerpc-ibm-aix6.1.0.0/10/include/c++/bits/atomic_base.h:35:10: fatal error: bits/c++config.h: No such file or directory
#   35 | #include <bits/c++config.h>
#      |          ^~~~~~~~~~~~~~~~~~
# compilation terminated.
ln -sf . %{buildroot}%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/include/c++/powerpc-ibm-aix%{aix_version}/pthread

cd %{buildroot}%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}


rm libatomic.so.%{libatomic_sover}
rm libstdc++.so.%{libstdcxx_sover}
rm libgomp.so.%{libgomp_sover}
rm libgfortran.so.%{libgfortran_sover}
rm libgcc_s.so.%{libgcc_sover}

rm libstdc++fs.a
rm libsupc++.a

mv pthread/libatomic.so.1.2.0 .
mv pthread/libstdc++.so.6.0.28 .
mv pthread/libgomp.so.1.0.0 .
mv pthread/libgfortran.so.5.0.0 .
mv pthread/libgcc_s.so.1 %{buildroot}%{_libdir}/libgcc_s.so.%{libgcc_sover}

mv pthread/libcaf_single.a .
mv pthread/libgfortran.spec .
mv pthread/libgomp.spec .

svr4-shrlib libatomic.so.1.2.0 libatomic.so.%{libatomic_sover} ppc64
svr4-shrlib libstdc++.so.6.0.28 libstdc++.so.%{libstdcxx_sover} ppc64
svr4-shrlib libgomp.so.1.0.0 libgomp.so.%{libgomp_sover} ppc64
svr4-shrlib libgfortran.so.5.0.0 libgfortran.so.%{libgfortran_sover} ppc64


mv libatomic.so.1.2.0 %{buildroot}%{_libdir}/libatomic.so.%{libatomic_sover}
mv libstdc++.so.6.0.28 %{buildroot}%{_libdir}/libstdc++.so.%{libstdcxx_sover}
mv libgomp.so.1.0.0 %{buildroot}%{_libdir}/libgomp.so.%{libgomp_sover}
mv libgfortran.so.5.0.0 %{buildroot}%{_libdir}/libgfortran.so.%{libgfortran_sover}

ln -sf -r %{buildroot}%{_libdir}/libgcc_s.so.%{libgcc_sover} libgcc_s.so
ln -sf -r %{buildroot}%{_libdir}/libatomic.so.%{libatomic_sover} libatomic.so
ln -sf -r %{buildroot}%{_libdir}/libstdc++.so.%{libstdcxx_sover} libstdc++.so
ln -sf -r %{buildroot}%{_libdir}/libgomp.so.%{libgomp_sover} libgomp.so
ln -sf -r %{buildroot}%{_libdir}/libgfortran.so.%{libgfortran_sover} libgfortran.so

rm -r pthread

cd %{buildroot}/%{_bindir}

# Remove the AIX-versioned binaries
rm powerpc-ibm-aix*
rm c++-%{major}

find %{buildroot}%{_libexecdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/install-tools/ -type f | xargs perl -pi -e 's|/bin/sh|%{_bindir}/bash|g'

rm -r %{buildroot}/%{_datadir}/gcc-%{major}/python
rm -r %{buildroot}/%{_mandir}/man7
rm -r %{buildroot}/%{_infodir}

# Remove the following crt* files, None of our specs refer to them
# and due to the --with-specs, these are all built as 64-bit
# so they're wrong anyway. These are intended to be 32-bit versions.
cd %{buildroot}%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/
rm crtcxa.o crtcxa_s.o crtdbase.o

%files
%defattr(-, qsys, *none)

%{_bindir}/gcc-%{major}
%{_bindir}/gcc-ar-%{major}
%{_bindir}/gcc-nm-%{major}
%{_bindir}/gcc-ranlib-%{major}
%{_bindir}/gcov-%{major}
%{_bindir}/gcov-dump-%{major}
%{_bindir}/gcov-tool-%{major}

%dir %{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}
%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/crtcxa_64.o
%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/crtcxa_64_s.o
%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/crtdbase_64.o
%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/libgcov.a
%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/libgcc.a
%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/libgcc_eh.a
%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/libgcc_s.so
%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/libatomic.so

%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/libgomp.so
%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/libgomp.spec

%dir %{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/include
%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/include/[^c++]*
%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/include-fixed
%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/install-tools

%{_libexecdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/collect2
%{_libexecdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/lto-wrapper

%dir %{_libexecdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/install-tools
%{_libexecdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/install-tools/fixinc.sh
%{_libexecdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/install-tools/fixincl
%{_libexecdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/install-tools/mkheaders
%{_libexecdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/install-tools/mkinstalldirs

%{_mandir}/man1/gcc-%{major}.1*
%{_mandir}/man1/gcov-%{major}.1*
%{_mandir}/man1/gcov-dump-%{major}.1*
%{_mandir}/man1/gcov-tool-%{major}.1*

%files -n gcc%{major}-cpp
%defattr(-, qsys, *none)
%{_bindir}/cpp-%{major}
%dir %{_libexecdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}
%{_libexecdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/cc1
%{_mandir}/man1/cpp-%{major}.1*

%files -n gcc%{major}-cplusplus
%defattr(-, qsys, *none)
%{_bindir}/g++-%{major}
%{_libexecdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/cc1plus
%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/libstdc++.so.6.0.28-gdb.py
%{_mandir}/man1/g++-%{major}.1*

%files -n libgcc_s%{libgcc_sover}
%defattr(-, qsys, *none)
%{_libdir}/libgcc_s.so.%{libgcc_sover}

%files -n libatomic%{libatomic_sover}
%defattr(-, qsys, *none)
%{_libdir}/libatomic.so.%{libatomic_sover}

%files -n gcc%{major}-gfortran
%defattr(-, qsys, *none)
%{_bindir}/gfortran-%{major}

%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/libgfortran.so

%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/finclude/

%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/libcaf_single.a
%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/libgfortran.spec

%{_libexecdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/f951

%{_mandir}/man1/gfortran-%{major}.1*


%files -n libgfortran%{libgfortran_sover}
%defattr(-, qsys, *none)
%{_libdir}/libgfortran.so.%{libgfortran_sover}

%files -n libgomp%{libgomp_sover}
%defattr(-, qsys, *none)
%{_libdir}/libgomp.so.%{libgomp_sover}

%files -n libstdcplusplus%{libstdcxx_sover}
%defattr(-, qsys, *none)
%{_libdir}/libstdc++.so.%{libstdcxx_sover}

%files -n libstdcplusplus%{libstdcxx_sover}-%{major}-devel
%defattr(-, qsys, *none)
%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/libstdc++.so
%{_libdir}/gcc/powerpc-ibm-aix%{aix_version}/%{major}/include/c++

%changelog
* Thu Nov 18 2021 Abdirahim Musse <abdirahim.musse@ibm.com> - 10.3.0
- Add libgfortran5 requires to gcc10-gfortran
- Add alias options -m64 to -maix64 and -m32 to -maix32 from upstream AIX patch

* Wed Nov 17 2021 Abdirahim Musse <abdirahim.musse@ibm.com> - 10.3.0
- Add POWER10 assembly invocation on PASE

* Tue Nov 16 2021 Abdirahim Musse <abdirahim.musse@ibm.com> - 10.3.0
- Add/Fixup intrapackage requires
- Add symlink to pthread multilib dir to resolve include files
- Ship crt*_64* files

* Wed Oct 06 2021 Kevin Adler <kadler@us.ibm.com> - 10.3.0
- Ensure we don't build with -G ld option to prevent blowing the TOC

* Thu Sep 30 2021 Kevin Adler <kadler@us.ibm.com> - 10.3.0
- Fix libgcc_s compatibility with GCC 6 libgfortran3

* Tue Sep 21 2021 Kevin Adler <kadler@us.ibm.com> - 10.3.0
- Add patch to work around linking failure for libgfortran
- Add code to repackage svr4 libgfortran

* Thu Aug 26 2021 Abdirahim Musse <abdirahim.musse@ibm.com> - 10.3.0
- Update to 10.3

* Fri Apr 09 2021 Kevin Adler <kadler@us.ibm.com> - 10.2.0
- Enable DWARF debugging support

* Fri Mar 26 2021 Kevin Adler <kadler@us.ibm.com> - 10.2.0
- Make GCC library packages upgrade GCC 6 versions
- Use only major version in paths, eg. /QOpenSys/pkgs/lib/gcc/10
- Fix binary suffix, eg. gcc10 -> gcc-10

* Thu Mar 4 2021 Xu Meng <mengxumx@cn.ibm.com>
- Clean up the spec file

* Fri Jan 8 2021 Xu Meng <mengxumx@cn.ibm.com>
- Initial package
